import { Client, Operations, QueryKey, QueryOptions, Query, Mutation } from './chunk-V5DEG2OQ.js';
import transformers, { camelCase } from '@kubb/core/transformers';
import path from 'node:path';
import { createPlugin, FileManager, PluginManager } from '@kubb/core';
import { renderTemplate } from '@kubb/core/utils';
import { createReactGenerator, pluginOasName, OperationGenerator } from '@kubb/plugin-oas';
import { pluginZodName } from '@kubb/plugin-zod';
import { useOperationManager } from '@kubb/plugin-oas/hooks';
import { pluginTsName } from '@kubb/plugin-ts';
import { useApp, File } from '@kubb/react';
import { jsxs, jsx } from '@kubb/react/jsx-runtime';

var clientGenerator = createReactGenerator({
  name: "client",
  Operation({ options, operation }) {
    const {
      plugin: {
        options: { output }
      }
    } = useApp();
    const { getSchemas, getName, getFile } = useOperationManager();
    const client = {
      name: getName(operation, { type: "function" }),
      file: getFile(operation)
    };
    const type = {
      file: getFile(operation, { pluginKey: [pluginTsName] }),
      schemas: getSchemas(operation, { pluginKey: [pluginTsName], type: "type" })
    };
    const zod = {
      file: getFile(operation, { pluginKey: [pluginZodName] }),
      schemas: getSchemas(operation, { pluginKey: [pluginZodName], type: "function" })
    };
    return /* @__PURE__ */ jsxs(File, { baseName: client.file.baseName, path: client.file.path, meta: client.file.meta, banner: output?.banner, footer: output?.footer, children: [
      /* @__PURE__ */ jsx(File.Import, { name: "client", path: options.importPath }),
      /* @__PURE__ */ jsx(File.Import, { name: ["RequestConfig"], path: options.importPath, isTypeOnly: true }),
      options.parser === "zod" && /* @__PURE__ */ jsx(File.Import, { extName: output?.extName, name: [zod.schemas.response.name], root: client.file.path, path: zod.file.path }),
      /* @__PURE__ */ jsx(
        File.Import,
        {
          extName: output?.extName,
          name: [
            type.schemas.request?.name,
            type.schemas.response.name,
            type.schemas.pathParams?.name,
            type.schemas.queryParams?.name,
            type.schemas.headerParams?.name,
            ...type.schemas.statusCodes?.map((item) => item.name) || []
          ].filter(Boolean),
          root: client.file.path,
          path: type.file.path,
          isTypeOnly: true
        }
      ),
      /* @__PURE__ */ jsx(
        Client,
        {
          name: client.name,
          baseURL: options.baseURL,
          dataReturnType: options.dataReturnType,
          pathParamsType: options.pathParamsType,
          typeSchemas: type.schemas,
          operation,
          parser: options.parser,
          zodSchemas: zod.schemas
        }
      )
    ] });
  }
});
var operationsGenerator = createReactGenerator({
  name: "client",
  Operations({ operations }) {
    const {
      pluginManager,
      plugin: {
        options: { output }
      }
    } = useApp();
    const name = "operations";
    const file = pluginManager.getFile({ name, extName: ".ts", pluginKey: [pluginClientName] });
    return /* @__PURE__ */ jsx(File, { baseName: file.baseName, path: file.path, meta: file.meta, banner: output?.banner, footer: output?.footer, children: /* @__PURE__ */ jsx(Operations, { name, operations }) });
  }
});

// ../plugin-client/src/plugin.ts
var pluginClientName = "plugin-client";
createPlugin((options) => {
  const {
    output = { path: "clients" },
    group,
    exclude = [],
    include,
    override = [],
    transformers: transformers2 = {},
    dataReturnType = "data",
    pathParamsType = "inline",
    operations = false,
    importPath = "@kubb/plugin-client/client",
    parser = "client"
  } = options;
  const template = group?.output ? group.output : `${output.path}/{{tag}}Controller`;
  return {
    name: pluginClientName,
    options: {
      output: {
        exportType: "barrelNamed",
        ...output
      },
      parser,
      dataReturnType,
      importPath,
      pathParamsType,
      baseURL: void 0
    },
    pre: [pluginOasName, parser === "zod" ? pluginZodName : void 0].filter(Boolean),
    resolvePath(baseName, pathMode, options2) {
      const root = path.resolve(this.config.root, this.config.output.path);
      const mode = pathMode ?? FileManager.getMode(path.resolve(root, output.path));
      if (mode === "single") {
        return path.resolve(root, output.path);
      }
      if (options2?.tag && group?.type === "tag") {
        const tag = camelCase(options2.tag);
        return path.resolve(root, renderTemplate(template, { tag }), baseName);
      }
      return path.resolve(root, output.path, baseName);
    },
    resolveName(name, type) {
      const resolvedName = camelCase(name, { isFile: type === "file" });
      if (type) {
        return transformers2?.name?.(resolvedName, type) || resolvedName;
      }
      return resolvedName;
    },
    async buildStart() {
      const [swaggerPlugin] = PluginManager.getDependedPlugins(this.plugins, [pluginOasName]);
      const oas = await swaggerPlugin.context.getOas();
      const root = path.resolve(this.config.root, this.config.output.path);
      const mode = FileManager.getMode(path.resolve(root, output.path));
      const baseURL = await swaggerPlugin.context.getBaseURL();
      const operationGenerator = new OperationGenerator(
        {
          ...this.plugin.options,
          baseURL
        },
        {
          oas,
          pluginManager: this.pluginManager,
          plugin: this.plugin,
          contentType: swaggerPlugin.context.contentType,
          exclude,
          include,
          override,
          mode
        }
      );
      const files = await operationGenerator.build(...[clientGenerator, operations ? operationsGenerator : void 0].filter(Boolean));
      await this.addFile(...files);
      if (this.config.output.exportType) {
        const barrelFiles = await this.fileManager.getBarrelFiles({
          root,
          output,
          files: this.fileManager.files,
          meta: {
            pluginKey: this.plugin.key
          },
          logger: this.logger
        });
        await this.addFile(...barrelFiles);
      }
    }
  };
});
var queryGenerator = createReactGenerator({
  name: "svelte-query",
  Operation({ options, operation }) {
    const {
      plugin: {
        options: { output }
      }
    } = useApp();
    const { getSchemas, getName, getFile } = useOperationManager();
    const isQuery = typeof options.query === "boolean" ? true : options.query?.methods.some((method) => operation.method === method);
    !isQuery && options.mutation && options.mutation.methods.some((method) => operation.method === method);
    const query = {
      name: getName(operation, { type: "function", prefix: "create" }),
      typeName: getName(operation, { type: "type" }),
      file: getFile(operation, { prefix: "create" })
    };
    const client = {
      name: getName(operation, { type: "function", pluginKey: [pluginClientName] })
    };
    const queryOptions = {
      name: transformers.camelCase(`${operation.getOperationId()} QueryOptions`)
    };
    const queryKey = {
      name: transformers.camelCase(`${operation.getOperationId()} QueryKey`),
      typeName: transformers.pascalCase(`${operation.getOperationId()} QueryKey`)
    };
    const type = {
      file: getFile(operation, { pluginKey: [pluginTsName] }),
      //todo remove type?
      schemas: getSchemas(operation, { pluginKey: [pluginTsName], type: "type" })
    };
    const zod = {
      file: getFile(operation, { pluginKey: [pluginZodName] }),
      schemas: getSchemas(operation, { pluginKey: [pluginZodName], type: "function" })
    };
    if (!isQuery || typeof options.query === "boolean") {
      return null;
    }
    return /* @__PURE__ */ jsxs(File, { baseName: query.file.baseName, path: query.file.path, meta: query.file.meta, banner: output?.banner, footer: output?.footer, children: [
      options.parser === "zod" && /* @__PURE__ */ jsx(File.Import, { extName: output?.extName, name: [zod.schemas.response.name], root: query.file.path, path: zod.file.path }),
      /* @__PURE__ */ jsx(File.Import, { name: ["createQuery", "queryOptions"], path: options.query.importPath }),
      /* @__PURE__ */ jsx(File.Import, { name: ["QueryKey", "WithRequired", "CreateBaseQueryOptions", "CreateQueryResult"], path: options.query.importPath, isTypeOnly: true }),
      /* @__PURE__ */ jsx(File.Import, { name: "client", path: options.client.importPath }),
      /* @__PURE__ */ jsx(File.Import, { name: ["RequestConfig"], path: options.client.importPath, isTypeOnly: true }),
      options.client.dataReturnType === "full" && /* @__PURE__ */ jsx(File.Import, { name: ["ResponseConfig"], path: options.client.importPath, isTypeOnly: true }),
      /* @__PURE__ */ jsx(
        File.Import,
        {
          extName: output?.extName,
          name: [
            type.schemas.request?.name,
            type.schemas.response.name,
            type.schemas.pathParams?.name,
            type.schemas.queryParams?.name,
            type.schemas.headerParams?.name,
            ...type.schemas.statusCodes?.map((item) => item.name) || []
          ].filter(Boolean),
          root: query.file.path,
          path: type.file.path,
          isTypeOnly: true
        }
      ),
      /* @__PURE__ */ jsx(
        QueryKey,
        {
          name: queryKey.name,
          typeName: queryKey.typeName,
          operation,
          pathParamsType: options.pathParamsType,
          typeSchemas: type.schemas,
          keysFn: options.query.key
        }
      ),
      /* @__PURE__ */ jsx(
        Client,
        {
          name: client.name,
          isExportable: false,
          isIndexable: false,
          baseURL: options.baseURL,
          operation,
          typeSchemas: type.schemas,
          zodSchemas: zod.schemas,
          dataReturnType: options.client.dataReturnType,
          pathParamsType: options.pathParamsType,
          parser: options.parser
        }
      ),
      /* @__PURE__ */ jsx(
        QueryOptions,
        {
          name: queryOptions.name,
          clientName: client.name,
          queryKeyName: queryKey.name,
          typeSchemas: type.schemas,
          pathParamsType: options.pathParamsType
        }
      ),
      /* @__PURE__ */ jsx(
        Query,
        {
          name: query.name,
          queryOptionsName: queryOptions.name,
          typeSchemas: type.schemas,
          pathParamsType: options.pathParamsType,
          operation,
          dataReturnType: options.client.dataReturnType,
          queryKeyName: queryKey.name,
          queryKeyTypeName: queryKey.typeName
        }
      )
    ] });
  }
});
var mutationGenerator = createReactGenerator({
  name: "svelte-query",
  Operation({ options, operation }) {
    const {
      plugin: {
        options: { output }
      }
    } = useApp();
    const { getSchemas, getName, getFile } = useOperationManager();
    const isQuery = typeof options.query === "boolean" ? true : options.query?.methods.some((method) => operation.method === method);
    const isMutation = !isQuery && options.mutation && options.mutation.methods.some((method) => operation.method === method);
    const mutation = {
      name: getName(operation, { type: "function", prefix: "create" }),
      typeName: getName(operation, { type: "type" }),
      file: getFile(operation, { prefix: "create" })
    };
    const type = {
      file: getFile(operation, { pluginKey: [pluginTsName] }),
      //todo remove type?
      schemas: getSchemas(operation, { pluginKey: [pluginTsName], type: "type" })
    };
    const zod = {
      file: getFile(operation, { pluginKey: [pluginZodName] }),
      schemas: getSchemas(operation, { pluginKey: [pluginZodName], type: "function" })
    };
    const client = {
      name: getName(operation, { type: "function", pluginKey: [pluginClientName] })
    };
    if (!isMutation || typeof options.mutation === "boolean") {
      return null;
    }
    return /* @__PURE__ */ jsxs(File, { baseName: mutation.file.baseName, path: mutation.file.path, meta: mutation.file.meta, banner: output?.banner, footer: output?.footer, children: [
      options.parser === "zod" && /* @__PURE__ */ jsx(File.Import, { extName: output?.extName, name: [zod.schemas.response.name], root: mutation.file.path, path: zod.file.path }),
      /* @__PURE__ */ jsx(File.Import, { name: ["createMutation"], path: options.mutation.importPath }),
      /* @__PURE__ */ jsx(File.Import, { name: ["CreateMutationOptions"], path: options.mutation.importPath, isTypeOnly: true }),
      /* @__PURE__ */ jsx(File.Import, { name: "client", path: options.client.importPath }),
      /* @__PURE__ */ jsx(File.Import, { name: ["RequestConfig", "ResponseConfig"], path: options.client.importPath, isTypeOnly: true }),
      /* @__PURE__ */ jsx(
        File.Import,
        {
          extName: output?.extName,
          name: [
            type.schemas.request?.name,
            type.schemas.response.name,
            type.schemas.pathParams?.name,
            type.schemas.queryParams?.name,
            type.schemas.headerParams?.name,
            ...type.schemas.statusCodes?.map((item) => item.name) || []
          ].filter(Boolean),
          root: mutation.file.path,
          path: type.file.path,
          isTypeOnly: true
        }
      ),
      /* @__PURE__ */ jsx(
        Client,
        {
          name: client.name,
          isExportable: false,
          isIndexable: false,
          baseURL: options.baseURL,
          operation,
          typeSchemas: type.schemas,
          zodSchemas: zod.schemas,
          dataReturnType: options.client.dataReturnType,
          pathParamsType: options.pathParamsType,
          parser: options.parser
        }
      ),
      /* @__PURE__ */ jsx(
        Mutation,
        {
          name: mutation.name,
          clientName: client.name,
          typeName: mutation.typeName,
          typeSchemas: type.schemas,
          operation,
          dataReturnType: options.client.dataReturnType,
          pathParamsType: options.pathParamsType
        }
      )
    ] });
  }
});

export { mutationGenerator, queryGenerator };
//# sourceMappingURL=chunk-QPJYCH77.js.map
//# sourceMappingURL=chunk-QPJYCH77.js.map