'use strict';

var react = require('@kubb/react');
var oas = require('@kubb/oas');
var utils = require('@kubb/core/utils');
var utils$1 = require('@kubb/plugin-oas/utils');
var jsxRuntime = require('@kubb/react/jsx-runtime');

// src/components/Mutation.tsx
function getParams({ pathParamsType, typeSchemas }) {
  return react.FunctionParams.factory({
    pathParams: {
      mode: pathParamsType === "object" ? "object" : "inlineSpread",
      children: utils$1.getPathParams(typeSchemas.pathParams, { typed: true })
    },
    data: typeSchemas.request?.name ? {
      type: typeSchemas.request?.name,
      optional: oas.isOptional(typeSchemas.request?.schema)
    } : void 0,
    params: typeSchemas.queryParams?.name ? {
      type: typeSchemas.queryParams?.name,
      optional: oas.isOptional(typeSchemas.queryParams?.schema)
    } : void 0,
    headers: typeSchemas.headerParams?.name ? {
      type: typeSchemas.headerParams?.name,
      optional: oas.isOptional(typeSchemas.headerParams?.schema)
    } : void 0,
    config: {
      type: typeSchemas.request?.name ? `Partial<RequestConfig<${typeSchemas.request?.name}>>` : "Partial<RequestConfig>",
      default: "{}"
    }
  });
}
function Client({
  name,
  isExportable = true,
  isIndexable = true,
  typeSchemas,
  baseURL,
  dataReturnType,
  parser,
  zodSchemas,
  pathParamsType,
  operation
}) {
  const path = new utils.URLPath(operation.path);
  const contentType = operation.getContentType();
  const isFormData = contentType === "multipart/form-data";
  const headers = [
    contentType !== "application/json" ? `'Content-Type': '${contentType}'` : void 0,
    typeSchemas.headerParams?.name ? "...headers" : void 0
  ].filter(Boolean);
  const generics = [
    typeSchemas.response.name,
    typeSchemas.errors?.map((item) => item.name).join(" | ") || "unknown",
    typeSchemas.request?.name || "unknown"
  ].filter(Boolean);
  const params = getParams({ pathParamsType, typeSchemas });
  const clientParams = react.FunctionParams.factory({
    config: {
      mode: "object",
      children: {
        method: {
          value: JSON.stringify(operation.method)
        },
        url: {
          value: path.template
        },
        baseURL: baseURL ? {
          value: JSON.stringify(baseURL)
        } : void 0,
        params: typeSchemas.queryParams?.name ? {} : void 0,
        data: typeSchemas.request?.name ? {
          value: isFormData ? "formData" : void 0
        } : void 0,
        headers: headers.length ? {
          value: headers.length ? `{ ${headers.join(", ")}, ...config.headers }` : void 0
        } : void 0,
        config: {
          mode: "inlineSpread"
        }
      }
    }
  });
  const formData = isFormData ? `
   const formData = new FormData()
   if(data) {
    Object.keys(data).forEach((key) => {
      const value = data[key];
      if (typeof key === "string" && (typeof value === "string" || value instanceof Blob)) {
        formData.append(key, value);
      }
    })
   }
  ` : "";
  return /* @__PURE__ */ jsxRuntime.jsx(react.File.Source, { name, isExportable, isIndexable, children: /* @__PURE__ */ jsxRuntime.jsxs(
    react.Function,
    {
      name,
      async: true,
      export: isExportable,
      params: params.toConstructor(),
      JSDoc: {
        comments: utils$1.getComments(operation)
      },
      children: [
        formData,
        `const res = await client<${generics.join(", ")}>(${clientParams.toCall()})`,
        /* @__PURE__ */ jsxRuntime.jsx("br", {}),
        dataReturnType === "full" && parser === "zod" && zodSchemas && `return {...res, data: ${zodSchemas.response.name}.parse(res.data)}`,
        dataReturnType === "data" && parser === "zod" && zodSchemas && `return ${zodSchemas.response.name}.parse(res.data)`,
        dataReturnType === "full" && parser === "client" && "return res",
        dataReturnType === "data" && parser === "client" && "return res.data"
      ]
    }
  ) });
}
Client.getParams = getParams;
function Operations({ name, operations }) {
  const operationsObject = {};
  operations.forEach((operation) => {
    operationsObject[operation.getOperationId()] = {
      path: new utils.URLPath(operation.path).URL,
      method: operation.method
    };
  });
  return /* @__PURE__ */ jsxRuntime.jsx(react.File.Source, { name, isExportable: true, isIndexable: true, children: /* @__PURE__ */ jsxRuntime.jsx(react.Const, { name, export: true, asConst: true, children: JSON.stringify(operationsObject, void 0, 2) }) });
}
function getParams2({ dataReturnType, typeSchemas }) {
  const mutateParams = react.FunctionParams.factory({
    ...utils$1.getPathParams(typeSchemas.pathParams, { typed: true }),
    data: typeSchemas.request?.name ? {
      type: typeSchemas.request?.name,
      optional: oas.isOptional(typeSchemas.request?.schema)
    } : void 0,
    params: typeSchemas.queryParams?.name ? {
      type: typeSchemas.queryParams?.name,
      optional: oas.isOptional(typeSchemas.queryParams?.schema)
    } : void 0,
    headers: typeSchemas.headerParams?.name ? {
      type: typeSchemas.headerParams?.name,
      optional: oas.isOptional(typeSchemas.headerParams?.schema)
    } : void 0
  });
  const TData = dataReturnType === "data" ? typeSchemas.response.name : `ResponseConfig<${typeSchemas.response.name}>`;
  const TRequest = mutateParams.toConstructor({ valueAsType: true });
  return react.FunctionParams.factory({
    options: {
      type: `
{
  mutation?: CreateMutationOptions<${[TData, typeSchemas.errors?.map((item) => item.name).join(" | ") || "unknown", `{${TRequest}`].join(", ")}>,
  client?: ${typeSchemas.request?.name ? `Partial<RequestConfig<${typeSchemas.request?.name}>>` : "Partial<RequestConfig>"},
}
`,
      default: "{}"
    }
  });
}
function Mutation({ name, clientName, pathParamsType, dataReturnType, typeSchemas, operation }) {
  const params = getParams2({
    pathParamsType,
    dataReturnType,
    typeSchemas
  });
  const clientParams = Client.getParams({
    typeSchemas,
    pathParamsType
  });
  const mutationParams = react.FunctionParams.factory({
    data: {
      // No use of pathParams because useMutation can only take one argument in object form,
      // see https://tanstack.com/query/latest/docs/framework/react/reference/useMutation#usemutation
      mode: "object",
      //TODO rename with value
      children: {
        ...utils$1.getPathParams(typeSchemas.pathParams, { typed: true }),
        data: typeSchemas.request?.name ? {
          type: typeSchemas.request?.name,
          optional: oas.isOptional(typeSchemas.request?.schema)
        } : void 0,
        params: typeSchemas.queryParams?.name ? {
          type: typeSchemas.queryParams?.name,
          optional: oas.isOptional(typeSchemas.queryParams?.schema)
        } : void 0,
        headers: typeSchemas.headerParams?.name ? {
          type: typeSchemas.headerParams?.name,
          optional: oas.isOptional(typeSchemas.headerParams?.schema)
        } : void 0
      }
    }
  });
  return /* @__PURE__ */ jsxRuntime.jsx(react.File.Source, { name, isExportable: true, isIndexable: true, children: /* @__PURE__ */ jsxRuntime.jsx(
    react.Function,
    {
      name,
      export: true,
      params: params.toConstructor(),
      JSDoc: {
        comments: utils$1.getComments(operation)
      },
      children: `
        const { mutation: mutationOptions, client: config = {} } = options ?? {}

        return createMutation({
          mutationFn: async(${mutationParams.toConstructor()}) => {
            return ${clientName}(${clientParams.toCall()})
          },
          ...mutationOptions
        })
    `
    }
  ) });
}
function getParams3({ pathParamsType, typeSchemas }) {
  return react.FunctionParams.factory({
    pathParams: {
      mode: pathParamsType === "object" ? "object" : "inlineSpread",
      children: utils$1.getPathParams(typeSchemas.pathParams, { typed: true })
    },
    data: typeSchemas.request?.name ? {
      type: typeSchemas.request?.name,
      optional: oas.isOptional(typeSchemas.request?.schema)
    } : void 0,
    params: typeSchemas.queryParams?.name ? {
      type: typeSchemas.queryParams?.name,
      optional: oas.isOptional(typeSchemas.queryParams?.schema)
    } : void 0
  });
}
function QueryKey({ name, typeSchemas, pathParamsType, operation, typeName, keysFn = (name2) => name2 }) {
  const path = new utils.URLPath(operation.path);
  const params = getParams3({ pathParamsType, typeSchemas });
  const keys = [
    path.toObject({
      type: "path",
      stringify: true
    }),
    typeSchemas.queryParams?.name ? "...(params ? [params] : [])" : void 0,
    typeSchemas.request?.name ? "...(data ? [data] : [])" : void 0
  ].filter(Boolean);
  return /* @__PURE__ */ jsxRuntime.jsxs(jsxRuntime.Fragment, { children: [
    /* @__PURE__ */ jsxRuntime.jsx(react.File.Source, { name, isExportable: true, isIndexable: true, children: /* @__PURE__ */ jsxRuntime.jsx(react.Function.Arrow, { name, export: true, params: params.toConstructor(), singleLine: true, children: `[${keysFn(keys).join(", ")}] as const` }) }),
    /* @__PURE__ */ jsxRuntime.jsx(react.File.Source, { name: typeName, isExportable: true, isIndexable: true, isTypeOnly: true, children: /* @__PURE__ */ jsxRuntime.jsx(react.Type, { name: typeName, export: true, children: `ReturnType<typeof ${name}>` }) })
  ] });
}
QueryKey.getParams = getParams3;
function getParams4({ pathParamsType, typeSchemas }) {
  return react.FunctionParams.factory({
    pathParams: {
      mode: pathParamsType === "object" ? "object" : "inlineSpread",
      children: utils$1.getPathParams(typeSchemas.pathParams, { typed: true })
    },
    data: typeSchemas.request?.name ? {
      type: typeSchemas.request?.name,
      optional: oas.isOptional(typeSchemas.request?.schema)
    } : void 0,
    params: typeSchemas.queryParams?.name ? {
      type: typeSchemas.queryParams?.name,
      optional: oas.isOptional(typeSchemas.queryParams?.schema)
    } : void 0,
    headers: typeSchemas.headerParams?.name ? {
      type: typeSchemas.headerParams?.name,
      optional: oas.isOptional(typeSchemas.headerParams?.schema)
    } : void 0,
    config: {
      type: typeSchemas.request?.name ? `Partial<RequestConfig<${typeSchemas.request?.name}>>` : "Partial<RequestConfig>",
      default: "{}"
    }
  });
}
function QueryOptions({ name, clientName, typeSchemas, pathParamsType, queryKeyName }) {
  const params = getParams4({ pathParamsType, typeSchemas });
  const clientParams = Client.getParams({
    typeSchemas,
    pathParamsType
  });
  const queryKeyParams = QueryKey.getParams({
    pathParamsType,
    typeSchemas
  });
  return /* @__PURE__ */ jsxRuntime.jsx(react.File.Source, { name, isExportable: true, isIndexable: true, children: /* @__PURE__ */ jsxRuntime.jsx(react.Function, { name, export: true, params: params.toConstructor(), children: `
      const queryKey = ${queryKeyName}(${queryKeyParams.toCall()})
      return queryOptions({
       queryKey,
       queryFn: async () => {
          return ${clientName}(${clientParams.toCall()})
       },
      })
` }) });
}
QueryOptions.getParams = getParams4;
function getParams5({ pathParamsType, dataReturnType, typeSchemas }) {
  const TData = dataReturnType === "data" ? typeSchemas.response.name : `ResponseConfig<${typeSchemas.response.name}>`;
  return react.FunctionParams.factory({
    pathParams: {
      mode: pathParamsType === "object" ? "object" : "inlineSpread",
      children: utils$1.getPathParams(typeSchemas.pathParams, { typed: true })
    },
    data: typeSchemas.request?.name ? {
      type: typeSchemas.request?.name,
      optional: oas.isOptional(typeSchemas.request?.schema)
    } : void 0,
    params: typeSchemas.queryParams?.name ? {
      type: typeSchemas.queryParams?.name,
      optional: oas.isOptional(typeSchemas.queryParams?.schema)
    } : void 0,
    headers: typeSchemas.headerParams?.name ? {
      type: typeSchemas.headerParams?.name,
      optional: oas.isOptional(typeSchemas.headerParams?.schema)
    } : void 0,
    options: {
      type: `
{
  query?: Partial<CreateBaseQueryOptions<${[TData, typeSchemas.errors?.map((item) => item.name).join(" | ") || "unknown", "TData", "TQueryData", "TQueryKey"].join(", ")}>>,
  client?: ${typeSchemas.request?.name ? `Partial<RequestConfig<${typeSchemas.request?.name}>>` : "Partial<RequestConfig>"}
}
`,
      default: "{}"
    }
  });
}
function Query({ name, queryKeyTypeName, queryOptionsName, queryKeyName, pathParamsType, dataReturnType, typeSchemas, operation }) {
  const TData = dataReturnType === "data" ? typeSchemas.response.name : `ResponseConfig<${typeSchemas.response.name}>`;
  const returnType = `CreateQueryResult<${["TData", typeSchemas.errors?.map((item) => item.name).join(" | ") || "unknown"].join(", ")}> & { queryKey: TQueryKey }`;
  const generics = [`TData = ${TData}`, `TQueryData = ${TData}`, `TQueryKey extends QueryKey = ${queryKeyTypeName}`];
  const queryKeyParams = QueryKey.getParams({
    pathParamsType,
    typeSchemas
  });
  const queryOptionsParams = QueryOptions.getParams({
    pathParamsType,
    typeSchemas
  });
  const params = getParams5({
    pathParamsType,
    dataReturnType,
    typeSchemas
  });
  const queryOptions = `${queryOptionsName}(${queryOptionsParams.toCall()}) as unknown as CreateBaseQueryOptions`;
  return /* @__PURE__ */ jsxRuntime.jsx(react.File.Source, { name, isExportable: true, isIndexable: true, children: /* @__PURE__ */ jsxRuntime.jsx(
    react.Function,
    {
      name,
      export: true,
      generics: generics.join(", "),
      params: params.toConstructor(),
      JSDoc: {
        comments: utils$1.getComments(operation)
      },
      children: `
       const { query: queryOptions, client: config = {} } = options ?? {}
       const queryKey = queryOptions?.queryKey ?? ${queryKeyName}(${queryKeyParams.toCall()})

       const query = createQuery({
        ...${queryOptions},
        queryKey,
        ...queryOptions as unknown as Omit<CreateBaseQueryOptions, "queryKey">
       }) as ${returnType}

       query.queryKey = queryKey as TQueryKey

       return query
       `
    }
  ) });
}
Query.getParams = getParams5;

exports.Client = Client;
exports.Mutation = Mutation;
exports.Operations = Operations;
exports.Query = Query;
exports.QueryKey = QueryKey;
exports.QueryOptions = QueryOptions;
//# sourceMappingURL=chunk-TD7H4OF4.cjs.map
//# sourceMappingURL=chunk-TD7H4OF4.cjs.map